@@ -31,11 +31,8 @@ import re
 import fnmatch
 import glob
 
-__version__ = '0.9.10'
-THIS_FILE_NAME = os.path.basename(__file__)
-
-def _is_windows():
-    return sys.platform.lower().startswith('win')
+__version__ = '0.9.13'
+PACKAGE_NAME = 'greplica'
 
 class BinaryDetectedException(Exception):
     def __init__(self, *args: object) -> None:
@@ -340,37 +337,45 @@ class AnsiString:
         return self._s
 
     @staticmethod
-    def _insert_settings_to_dict(settings_dict, idx, apply, settings):
+    def _insert_settings_to_dict(settings_dict, idx, apply, settings, topmost=True):
         if idx not in settings_dict:
             settings_dict[idx] = [[] for _ in range(__class__.SETTINGS_ITEM_LIST_LEN)]
         list_idx = __class__.SETTINGS_APPLY_IDX if apply else __class__.SETTINGS_REMOVE_IDX
-        settings_dict[idx][list_idx].append(settings)
+        if topmost:
+            settings_dict[idx][list_idx].append(settings)
+        else:
+            settings_dict[idx][list_idx].insert(0, settings)
 
-    def _insert_settings(self, idx, apply, settings):
-        __class__._insert_settings_to_dict(self._color_settings, idx, apply, settings)
+    def _insert_settings(self, idx, apply, settings, topmost=True):
+        __class__._insert_settings_to_dict(self._color_settings, idx, apply, settings, topmost)
 
-    def apply_formatting(self, setting_or_settings, start_idx=0, length=None):
+    def apply_formatting(self, setting_or_settings, start_idx=0, length=None, topmost=True):
         '''
         Sets the formatting for a given range of characters.
         Inputs: setting_or_settings - Can either be a single item or list of items;
                                       each item can either be a string or AnsiFormat type
                 start_idx - The string start index where setting(s) are to be applied
                 length - Number of characters to apply settings or None to apply until end of string
+                topmost - When true, this setting is placed at the end of the set for the given
+                        start_index meaning it is applied last; when false, setting is applied first
 
         Note: The desired effect may not be achieved if the same setting is applied over an
               overlapping range of characters.
         '''
-        if length is not None and length <= 0:
+        if not setting_or_settings:
+            # Ignore - nothing to apply
+            return
+        elif length is not None and length <= 0:
             raise ValueError('Invalid length {}'.format(length))
 
         settings = __class__.Settings(setting_or_settings)
 
         # Apply settings
-        self._insert_settings(start_idx, True, settings)
+        self._insert_settings(start_idx, True, settings, topmost)
 
         if length is not None:
             # Remove settings
-            self._insert_settings(start_idx + length, False, settings)
+            self._insert_settings(start_idx + length, False, settings, topmost)
 
     def apply_formatting_for_match(self, setting_or_settings, match_object, group=0):
         '''
@@ -502,7 +507,7 @@ class AnsiString:
                         format_settings_strs.append(ansi_format.value)
                 format_settings = __class__.Settings(';'.join(format_settings_strs))
 
-            __class__._insert_settings_to_dict(settings_dict, 0, True, format_settings)
+            __class__._insert_settings_to_dict(settings_dict, 0, True, format_settings, True)
 
         clear_needed = False
         for idx, settings, current_settings in __class__.SettingsIterator(settings_dict):
@@ -1095,10 +1100,11 @@ class Grep:
             colors = os.environ['GREP_COLORS'].split(':')
             for color in colors:
                 key_val = color.split('=', maxsplit=1)
-                if len(key_val) == 2 and key_val[0] in grep_color_dict:
+                if key_val[0] in grep_color_dict:
                     if isinstance(grep_color_dict[key_val[0]], bool):
-                        grep_color_dict[key_val[0]] = key_val[1].lower() in ['1', 't', 'true', 'on']
-                    else:
+                        # Set the value to True when specified at all
+                        grep_color_dict[key_val[0]] = True
+                    elif len(key_val) == 2:
                         # The string must be integers separated by semicolon
                         is_valid = True
                         for item in key_val[1].split(';'):
@@ -1128,6 +1134,8 @@ class Grep:
             self.fixed_string_parse = False
             self.color_enabled = False
             self.matching_color = None
+            self.matching_line_color = None
+            self.context_line_color = None
             self.file = None
             self.file_iter = None
             self.line_data_dict = {}
@@ -1159,6 +1167,7 @@ class Grep:
             self.current_after_context_counter = 0
             self.binary_detected = False
             self.line_num = 0
+            self.byte_offset = 0
             self.num_matches = 0
             self.file = file
             if file:
@@ -1249,6 +1258,8 @@ class Grep:
             if not line_slices:
                 # Default to printing the entire line
                 line_slices = [slice(0, None)]
+            else:
+                line_slices.sort(key=lambda x: x.start)
 
             for line_slice in line_slices:
                 if line_slice.start is not None:
@@ -1268,8 +1279,12 @@ class Grep:
             Called when parsing of line is complete.
             Inputs: is_match - True iff the current line is a match
             '''
+
             if is_match:
                 self.num_matches += 1
+                self.formatted_line.apply_formatting(self.matching_line_color, topmost=False)
+            else:
+                self.formatted_line.apply_formatting(self.context_line_color, topmost=False)
 
             if (
                 (is_match or  (self.current_after_context_counter > 0))
@@ -1312,7 +1327,14 @@ class Grep:
         return AutoInputFileIterable(path, 'rb', self.end)
 
     def _generate_line_format(self, grep_color_dict, name_num_sep, name_byte_sep, result_sep):
+        if grep_color_dict and 'se' in grep_color_dict:
+            se_color = grep_color_dict['se']
+            name_num_sep = str(AnsiString(name_num_sep, se_color))
+            name_byte_sep = str(AnsiString(name_byte_sep, se_color))
+            result_sep = str(AnsiString(result_sep, se_color))
+
         line_format = ''
+
         output_file_name_only = (
             self._print_count_only
             or self._print_matching_files_only
@@ -1394,19 +1416,6 @@ class Grep:
         else:
             data.files = [self._make_file_iterable(f) for f in self._files]
 
-        if data.color_enabled:
-            grep_color_dict = __class__._generate_color_dict()
-            data.matching_color = grep_color_dict['mt']
-            if data.matching_color is None:
-                if self.invert_match:
-                    # I don't get this setting because matching line isn't colored in this case
-                    data.matching_color = grep_color_dict['mc']
-                else:
-                    data.matching_color = grep_color_dict['ms']
-        else:
-            grep_color_dict = None
-            data.matching_color = None
-
         data.expressions = self._expressions
 
         for i in range(len(data.expressions)):
@@ -1436,35 +1445,43 @@ class Grep:
         else:
             data.fixed_string_parse = (self._search_type == __class__.SearchType.FIXED_STRINGS)
 
-        if data.color_enabled and grep_color_dict['se']:
-            data.line_format = self._generate_line_format(
-                grep_color_dict,
-                str(AnsiString(self._name_num_sep, grep_color_dict['se'])),
-                str(AnsiString(self._name_byte_sep, grep_color_dict['se'])),
-                str(AnsiString(self._results_sep, grep_color_dict['se']))
-            )
-            data.context_line_format = self._generate_line_format(
-                grep_color_dict,
-                str(AnsiString(self._context_name_num_sep, grep_color_dict['se'])),
-                str(AnsiString(self._context_name_byte_sep, grep_color_dict['se'])),
-                str(AnsiString(self._context_results_sep, grep_color_dict['se']))
-            )
+        if data.color_enabled:
+            grep_color_dict = __class__._generate_color_dict()
+            data.matching_color = grep_color_dict['mt']
+            if data.matching_color is None:
+                if self.invert_match:
+                    # I don't get this setting because matching line isn't colored in this case
+                    data.matching_color = grep_color_dict['mc']
+                else:
+                    data.matching_color = grep_color_dict['ms']
+            if grep_color_dict['rv'] and self._invert_match:
+                data.matching_line_color = grep_color_dict['cx']
+                data.context_line_color = grep_color_dict['sl']
+            else:
+                data.matching_line_color = grep_color_dict['sl']
+                data.context_line_color = grep_color_dict['cx']
             data.context_sep = str(AnsiString(self._context_sep, grep_color_dict['se']))
         else:
-            data.line_format = self._generate_line_format(
-                grep_color_dict,
-                self._name_num_sep,
-                self._name_byte_sep,
-                self._results_sep
-            )
-            data.context_line_format = self._generate_line_format(
-                grep_color_dict,
-                self._context_name_num_sep,
-                self._context_name_byte_sep,
-                self._context_results_sep
-            )
+            grep_color_dict = None
+            data.matching_color = None
+            data.matching_line_color = None
+            data.context_line_color = None
             data.context_sep = self._context_sep
 
+        data.line_format = self._generate_line_format(
+            grep_color_dict,
+            self._name_num_sep,
+            self._name_byte_sep,
+            self._results_sep
+        )
+
+        data.context_line_format = self._generate_line_format(
+            grep_color_dict,
+            self._context_name_num_sep,
+            self._context_name_byte_sep,
+            self._context_results_sep
+        )
+
         if (
             not self._quiet
             and not self._print_count_only
@@ -1488,14 +1505,17 @@ class Grep:
         '''
         match_found = False
         for expression in data.expressions:
-            if not expression:
+            if match_found and not data.color_enabled and not self.only_matching:
+                # No need to keep looping through expressions
+                break
+            elif not expression:
                 # Special case: always a match on empty string, and nothing to color
                 match_found = True
             elif data.fixed_string_parse:
                 loc = data.line.find(expression)
                 if loc >= 0:
-                    match_found = not self._invert_match
-                    if match_found and (data.color_enabled or self.only_matching):
+                    match_found = True
+                    if data.color_enabled or self.only_matching:
                         while loc >= 0:
                             if data.color_enabled:
                                 data.formatted_line.apply_formatting(
@@ -1503,9 +1523,6 @@ class Grep:
                             if self.only_matching:
                                 data.line_slices.append(slice(loc, loc + len(expression)))
                             loc = data.line.find(expression, loc + len(expression))
-                elif self._invert_match:
-                    # Color setting is ignored in this case - just print it
-                    match_found = True
             else:
                 # Regular expression matching
                 flags = 0
@@ -1514,27 +1531,21 @@ class Grep:
                 if self._line_regexp:
                     m = re.fullmatch(expression, data.line, flags)
                     if m is not None:
-                        match_found = not self._invert_match
+                        match_found = True
                         if data.color_enabled:
                             # This is going to just format the whole line
                             data.formatted_line.apply_formatting_for_match(data.matching_color, m)
                         if self.only_matching:
                             data.line_slices.append(slice(m.start(0), m.end(0)))
-                    elif self._invert_match:
-                        # Color setting is ignored in this case - just print it
-                        match_found = True
                 else:
-                    line_matches = False
                     for m in re.finditer(expression, data.line, flags):
-                        line_matches = True
-                        match_found = not self._invert_match
+                        match_found = True
                         if data.color_enabled:
                             data.formatted_line.apply_formatting_for_match(data.matching_color, m)
                         if self.only_matching:
                             data.line_slices.append(slice(m.start(0), m.end(0)))
-                    if self._invert_match and not line_matches:
-                        # Color setting is ignored in this case - just print it
-                        match_found = True
+        if self._invert_match:
+            match_found = not match_found
         data.parse_complete(match_found)
         return match_found
 
@@ -1566,7 +1577,7 @@ class Grep:
         except EnvironmentError as ex:
             # This occurs if permission is denied
             if not self._no_messages:
-                print('{}: {}'.format(THIS_FILE_NAME, str(ex)), file=sys.stderr)
+                print('{}: {}'.format(PACKAGE_NAME, str(ex)), file=sys.stderr)
         else:
             try:
                 while data.next_line() and (self._max_count is None or data.num_matches < self._max_count):
@@ -1618,38 +1629,27 @@ class Grep:
         matched_files = []
 
         for file in data.files:
-            if _is_windows() and isinstance(file, AutoInputFileIterable):
-                # Need to manually expand this out
-                expanded_files = [self._make_file_iterable(f) for f in glob.glob(file.name)]
-                if not expanded_files:
-                    if not self._no_messages:
-                        print('No match for: {}'.format(file.name), file=sys.stderr)
-                    continue
+            if os.path.isdir(file.name):
+                if not self._is_excluded_dir(file.name):
+                    if self._directory_handling_type == __class__.Directory.READ:
+                        print('{}: {}: Is a directory'.format(PACKAGE_NAME, file.name))
+                    elif (
+                        self._directory_handling_type == __class__.Directory.RECURSE
+                        or self._directory_handling_type == __class__.Directory.RECURSE_LINKS
+                    ):
+                        followlinks = (self._directory_handling_type == __class__.Directory.RECURSE_LINKS)
+                        for root, dirs, recurse_files in os.walk(file.name, followlinks=followlinks):
+                            if not self._is_excluded_dir(root):
+                                for recurse_file in recurse_files:
+                                    file_path = os.path.join(root, recurse_file)
+                                    if self._parse_file(self._make_file_iterable(file_path), data):
+                                        matched_files += [file_path]
+                            else:
+                                # Do nothing and exclude anything that follows
+                                dirs[:] = []
             else:
-                # *nix and *nix based systems do this from command line
-                expanded_files = [file]
-            for expanded_file in expanded_files:
-                if os.path.isdir(expanded_file.name):
-                    if not self._is_excluded_dir(expanded_file.name):
-                        if self._directory_handling_type == __class__.Directory.READ:
-                            print('{}: {}: Is a directory'.format(THIS_FILE_NAME, expanded_file.name))
-                        elif (
-                            self._directory_handling_type == __class__.Directory.RECURSE
-                            or self._directory_handling_type == __class__.Directory.RECURSE_LINKS
-                        ):
-                            followlinks = (self._directory_handling_type == __class__.Directory.RECURSE_LINKS)
-                            for root, dirs, recurse_files in os.walk(expanded_file.name, followlinks=followlinks):
-                                if not self._is_excluded_dir(root):
-                                    for recurse_file in recurse_files:
-                                        file_path = os.path.join(root, recurse_file)
-                                        if self._parse_file(self._make_file_iterable(file_path), data):
-                                            matched_files += [file_path]
-                                else:
-                                    # Do nothing and exclude anything that follows
-                                    dirs[:] = []
-                else:
-                    if self._parse_file(expanded_file, data):
-                        matched_files += [expanded_file.name]
+                if self._parse_file(file, data):
+                    matched_files += [file.name]
 
         return matched_files
 
@@ -1659,29 +1659,38 @@ class GrepArgParser:
     '''
     def __init__(self):
         self._parser = argparse.ArgumentParser(
-            prog=THIS_FILE_NAME,
+            prog=PACKAGE_NAME,
             description='Reimplementation of grep command entirely in Python.',
             add_help=False)
+        self._parser.register('action', 'extend', __class__.ExtendArgparseAction)
+
         self._parser.add_argument('expressions_positional', type=str, nargs='?', default=None, metavar='EXPRESSIONS',
                             help='Expressions to search for, separated by newline character (\\n). '
                             'This is required if --regexp or --file are not specified.')
         self._parser.add_argument('file', type=str, nargs='*', default=[], metavar='FILE',
-                            help='Files or directories to search. Stdin will be searched if not specified. '
-                            'How directories are handled is controled by -d and -r options.')
+                            help='Files or directories to search. Stdin will be searched if not specified, '
+                            ' unless -r is specified. Then current directory will be recursively searched.'
+                            'How directories are handled is controlled by -d and -r options.')
 
         regexp_group = self._parser.add_argument_group('Expression Interpretation')
         regexp_type = regexp_group.add_mutually_exclusive_group()
         regexp_type.add_argument('-E', '--extended-regexp', action='store_true',
-                                help='EXPRESSIONS are extended regular expressions')
+                                help='EXPRESSIONS are "extended" regular expressions\n'
+                                'In this mode, greplica passes regular expressions directly to Python re '
+                                'without modification. This for the most part matches original "extended" '
+                                'syntax, but be aware that there may be differences.')
         regexp_type.add_argument('-F', '--fixed-strings', action='store_true',
                                 help='EXPRESSIONS are strings')
         regexp_type.add_argument('-G', '--basic-regexp', action='store_true',
-                                help='EXPRESSIONS are basic regular expressions')
+                                help='EXPRESSIONS are "basic" regular expressions\n'
+                                'In this mode, greplica modifies escaping sequences for characters ?+{}|() '
+                                'before passing to Python re. This for the most part matches original "basic" '
+                                'syntax, but be aware that there may be differences.')
         regexp_group.add_argument('-e', '--regexp', dest='expressions_option', metavar='EXPRESSIONS', type=str,
                                 default=None,
                                 help='use EXPRESSIONS for matching')
         regexp_group.add_argument('-f', '--file', metavar='FILE', dest='expressions_file', nargs='+',
-                                  action='append', default=[], type=str,
+                                  action='extend', default=[], type=str,
                                   help='take EXPRESSIONS from FILE')
         regexp_group.add_argument('-i', '--ignore-case', action='store_true',
                                 help='ignore case in expressions')
@@ -1728,13 +1737,13 @@ class GrepArgParser:
                                      'ACTION is \'read\', \'recurse\', or \'skip\'')
         output_ctrl_grp.add_argument('-r', '--recursive', action='store_true', help='same as --directories=recurse')
         output_ctrl_grp.add_argument('-R', '--dereference-recursive', action='store_true', help='same as --directories=recurse_links')
-        output_ctrl_grp.add_argument('--include', type=str, nargs='+', metavar='GLOB', action='append', default=[],
+        output_ctrl_grp.add_argument('--include', type=str, nargs='+', metavar='GLOB', action='extend', default=[],
                                      help='limit files to those matching GLOB')
-        output_ctrl_grp.add_argument('--exclude', type=str, nargs='+', metavar='GLOB', action='append', default=[],
+        output_ctrl_grp.add_argument('--exclude', type=str, nargs='+', metavar='GLOB', action='extend', default=[],
                                      help='skip files that match GLOB')
-        output_ctrl_grp.add_argument('--exclude-from', type=str, nargs='+', metavar='FILE', action='append', default=[],
+        output_ctrl_grp.add_argument('--exclude-from', type=str, nargs='+', metavar='FILE', action='extend', default=[],
                                      help='read FILE for exclude globs file name globs')
-        output_ctrl_grp.add_argument('--exclude-dir', type=str, nargs='+', metavar='GLOB', action='append', default=[],
+        output_ctrl_grp.add_argument('--exclude-dir', type=str, nargs='+', metavar='GLOB', action='extend', default=[],
                                      help='skip directories that match GLOB')
         output_ctrl_grp.add_argument('-L', '--files-without-match', action='store_true', help='print only names of FILEs with no selected lines')
         output_ctrl_grp.add_argument('-l', '--files-with-matches', action='store_true', help='print only names of FILEs with selected lines')
@@ -1771,54 +1780,70 @@ class GrepArgParser:
                                     'WHEN is \'always\', \'never\', or \'auto\'')
         context_ctrl_grp.add_argument('-U', '--binary', action='store_true', help='do not strip CR characters at EOL (MSDOS/Windows)')
 
+        self._args = None
+
+    class ExtendArgparseAction(argparse.Action):
+        def __call__(self, parser, namespace, values, option_string=None):
+            items = getattr(namespace, self.dest, [])
+            items.extend(values)
+            setattr(namespace, self.dest, items)
+
+    @staticmethod
+    def _is_windows():
+        return sys.platform.lower().startswith('win')
+
+    def _expand_cli_path(self, path):
+        if __class__._is_windows():
+            # Need to manually expand this out
+            expanded_paths = [f for f in glob.glob(path)]
+            if not expanded_paths:
+                if not self._args.no_messages:
+                    print('No match for: {}'.format(path), file=sys.stderr)
+        else:
+            # *nix and *nix based systems do this from command line
+            expanded_paths = [path]
+        return expanded_paths
+
+    def _expand_cli_paths(self, paths):
+        return [y for x in paths for y in self._expand_cli_path(x)]
+
     def parse(self, cliargs, grep_object:Grep):
         '''
         Parses command line arguments into the given Grep object
         '''
         grep_object.reset()
-        args = self._parser.parse_args(cliargs)
+        self._args = self._parser.parse_args(cliargs)
 
-        if not args:
+        if not self._args:
             return False
 
-        if args.version:
-            print('{} {}'.format(THIS_FILE_NAME, __version__))
+        if self._args.version:
+            print('{} {}'.format(PACKAGE_NAME, __version__))
             sys.exit(0)
 
         # Pars expressions from all of the different options into a single list of expressions
         expressions = []
-        if args.expressions_option is not None:
+        if self._args.expressions_option is not None:
             # Set expressions to the option
-            expressions.extend(_parse_expressions(args.expressions_option))
+            expressions.extend(_parse_expressions(self._args.expressions_option))
             # The first positional (expressions_positional) is a file
-            if args.expressions_positional is not None:
-                args.file.insert(0, args.expressions_positional)
-        elif args.expressions_file:
-            for file_group in args.expressions_file:
-                for file in file_group:
-                    if _is_windows():
-                        # Need to manually expand this out
-                        expanded_files = [f for f in glob.glob(file)]
-                        if not expanded_files:
-                            if not args.no_messages:
-                                print('No match for: {}'.format(file), file=sys.stderr)
-                            continue
-                    else:
-                        # *nix and *nix based systems do this from command line
-                        expanded_files = [file]
-                    for expressions_file in expanded_files:
-                        try:
-                            with open(expressions_file, 'r') as fp:
-                                expressions.extend(_parse_expressions(fp.read()))
-                        except EnvironmentError as ex:
-                            if not args.no_messages:
-                                print('{}: {}'.format(THIS_FILE_NAME, str(ex)), file=sys.stderr)
+            if self._args.expressions_positional is not None:
+                self._args.file.insert(0, self._args.expressions_positional)
+        elif self._args.expressions_file:
+            for file_group in self._args.expressions_file:
+                for file in self._expand_cli_paths(file_group):
+                    try:
+                        with open(file, 'r') as fp:
+                            expressions.extend(_parse_expressions(fp.read()))
+                    except EnvironmentError as ex:
+                        if not self._args.no_messages:
+                            print('{}: {}'.format(PACKAGE_NAME, str(ex)), file=sys.stderr)
             # The first positional (expressions_positional) is a file
-            if args.expressions_positional is not None:
-                args.file.insert(0, args.expressions_positional)
-        elif args.expressions_positional is not None:
+            if self._args.expressions_positional is not None:
+                self._args.file.insert(0, self._args.expressions_positional)
+        elif self._args.expressions_positional is not None:
             # Set expressions to the positional
-            expressions.extend(_parse_expressions(args.expressions_positional))
+            expressions.extend(_parse_expressions(self._args.expressions_positional))
 
         if not expressions:
             self._parser.print_usage()
@@ -1827,136 +1852,120 @@ class GrepArgParser:
 
         grep_object.add_expression(expressions)
 
-        if args.file:
-            grep_object.add_files(args.file)
+        if self._args.file:
+            grep_object.add_files(self._expand_cli_paths(self._args.file))
 
-        if args.extended_regexp:
+        if self._args.extended_regexp:
             grep_object.search_type = Grep.SearchType.EXTENDED_REGEXP
-        elif args.fixed_strings:
+        elif self._args.fixed_strings:
             grep_object.search_type = Grep.SearchType.FIXED_STRINGS
         else:
             # Basic regexp is default if no type specified
             grep_object.search_type = Grep.SearchType.BASIC_REGEXP
 
-        grep_object.ignore_case = args.ignore_case
-        grep_object.word_regexp = args.word_regexp
-        grep_object.line_regexp = args.line_regexp
-        grep_object.no_messages = args.no_messages
-        grep_object.invert_match = args.invert_match
-        grep_object.max_count = args.max_count
-        grep_object.output_line_numbers = args.line_number
-        grep_object.output_file_name = args.with_filename
-        grep_object.output_byte_offset = args.byte_offset
-        grep_object.line_buffered = args.line_buffered
-        grep_object.label = args.label
-        grep_object.quiet = args.quiet
-        grep_object.only_matching = args.only_matching
-        grep_object.strip_cr = not args.binary
-        grep_object.print_matching_files_only = args.files_with_matches
-        grep_object.print_non_matching_files_only = args.files_without_match
-        grep_object.print_count_only = args.count
+        grep_object.ignore_case = self._args.ignore_case
+        grep_object.word_regexp = self._args.word_regexp
+        grep_object.line_regexp = self._args.line_regexp
+        grep_object.no_messages = self._args.no_messages
+        grep_object.invert_match = self._args.invert_match
+        grep_object.max_count = self._args.max_count
+        grep_object.output_line_numbers = self._args.line_number
+        grep_object.output_file_name = self._args.with_filename
+        grep_object.output_byte_offset = self._args.byte_offset
+        grep_object.line_buffered = self._args.line_buffered
+        grep_object.label = self._args.label
+        grep_object.quiet = self._args.quiet
+        grep_object.only_matching = self._args.only_matching
+        grep_object.strip_cr = not self._args.binary
+        grep_object.print_matching_files_only = self._args.files_with_matches
+        grep_object.print_non_matching_files_only = self._args.files_without_match
+        grep_object.print_count_only = self._args.count
 
-        if args.context is not None:
-            grep_object.before_context_count = args.context
-            grep_object.after_context_count = args.context
+        if self._args.context is not None:
+            grep_object.before_context_count = self._args.context
+            grep_object.after_context_count = self._args.context
         else:
-            if args.before_context is not None:
-                grep_object.before_context_count = args.before_context
-            if args.after_context is not None:
-                grep_object.after_context_count = args.after_context
+            if self._args.before_context is not None:
+                grep_object.before_context_count = self._args.before_context
+            if self._args.after_context is not None:
+                grep_object.after_context_count = self._args.after_context
 
-        if args.recursive or args.directories == 'recurse':
+        if self._args.recursive or self._args.directories == 'recurse':
             grep_object.directory_handling_type = Grep.Directory.RECURSE
-            if not args.no_filename:
+            if not self._args.no_filename:
                 # Force output of file name
                 grep_object.output_file_name = True
-        elif args.dereference_recursive or args.directories == 'recurse_links':
+        elif self._args.dereference_recursive or self._args.directories == 'recurse_links':
             grep_object.directory_handling_type = Grep.Directory.RECURSE_LINKS
-            if not args.no_filename:
+            if not self._args.no_filename:
                 # Force output of file name
                 grep_object.output_file_name = True
-        elif args.directories == 'skip':
+        elif self._args.directories == 'skip':
             grep_object.directory_handling_type = Grep.Directory.SKIP
         else:
             grep_object.directory_handling_type = Grep.Directory.READ
 
-        if args.null_data:
+        if self._args.null_data:
             grep_object.end = b'\x00'
         else:
-            grep_object.end = bytes(args.end, "utf-8").decode("unicode_escape")
+            grep_object.end = bytes(self._args.end, "utf-8").decode("unicode_escape")
 
-        grep_object.results_sep = bytes(args.result_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.results_sep = bytes(self._args.result_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.results_sep += '\t'
-        if args.null:
+        if self._args.null:
             grep_object.results_sep += '\0'
-        grep_object.name_num_sep = bytes(args.name_num_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.name_num_sep = bytes(self._args.name_num_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.name_num_sep += '\t'
-        grep_object.name_byte_sep = bytes(args.name_byte_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.name_byte_sep = bytes(self._args.name_byte_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.name_byte_sep += '\t'
 
-        grep_object.context_sep = bytes(args.context_group_sep, "utf-8").decode("unicode_escape")
-        grep_object.context_results_sep = bytes(args.context_result_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.context_sep = bytes(self._args.context_group_sep, "utf-8").decode("unicode_escape")
+        grep_object.context_results_sep = bytes(self._args.context_result_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.context_results_sep += '\t'
-        if args.null:
+        if self._args.null:
             grep_object.context_results_sep += '\0'
-        grep_object.context_name_num_sep = bytes(args.context_name_num_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.context_name_num_sep = bytes(self._args.context_name_num_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.context_name_num_sep += '\t'
-        grep_object.context_name_byte_sep = bytes(args.context_name_byte_sep, "utf-8").decode("unicode_escape")
-        if args.initial_tab:
+        grep_object.context_name_byte_sep = bytes(self._args.context_name_byte_sep, "utf-8").decode("unicode_escape")
+        if self._args.initial_tab:
             grep_object.context_name_byte_sep += '\t'
 
-        if args.color == 'always':
+        if self._args.color == 'always':
             grep_object.color_output_mode = Grep.ColorOutputMode.ALWAYS
-        elif args.color == 'never':
+        elif self._args.color == 'never':
             grep_object.color_output_mode = Grep.ColorOutputMode.NEVER
         else:
             grep_object.color_output_mode = Grep.ColorOutputMode.AUTO
 
-        if args.binary_files == 'text' or args.text:
+        if self._args.binary_files == 'text' or self._args.text:
             grep_object.binary_parse_function = Grep.BinaryParseFunction.IGNORE_DECODE_ERRORS
-        elif args.binary_files == 'without-match' or args.binary_without_match:
+        elif self._args.binary_files == 'without-match' or self._args.binary_without_match:
             grep_object.binary_parse_function = Grep.BinaryParseFunction.SKIP
         else:
             grep_object.binary_parse_function = Grep.BinaryParseFunction.PRINT_ERROR
 
-        for include_glob in args.include:
-            grep_object.add_file_include_globs(include_glob)
+        grep_object.add_file_include_globs(self._args.include)
+        grep_object.add_file_exclude_globs(self._args.exclude)
 
-        for exclude_glob in args.exclude:
-            grep_object.add_file_exclude_globs(exclude_glob)
+        for exclude_file in self._expand_cli_paths(self._args.exclude_from):
+            try:
+                with open(exclude_file, 'r') as fp:
+                    for line in fp.readlines():
+                        if line.endswith('\n'):
+                            line = line[:-1]
+                        if line.endswith('\r'):
+                            line = line[:-1]
+                        grep_object.add_file_exclude_globs(line)
+            except EnvironmentError as ex:
+                if not self._args.no_messages:
+                    print('{}: {}'.format(PACKAGE_NAME, str(ex)), file=sys.stderr)
 
-        for exclude_files in args.exclude_from:
-            for exclude_file in exclude_files:
-                if _is_windows():
-                    # Need to manually expand this out
-                    expanded_files = [f for f in glob.glob(exclude_file)]
-                    if not expanded_files:
-                        if not args.no_messages:
-                            print('No match for: {}'.format(exclude_file), file=sys.stderr)
-                        continue
-                else:
-                    # *nix and *nix based systems do this from command line
-                    expanded_files = [exclude_file]
-                for expanded_file in expanded_files:
-                    try:
-                        with open(expanded_file, 'r') as fp:
-                            for line in fp.readlines():
-                                if line.endswith('\n'):
-                                    line = line[:-1]
-                                if line.endswith('\r'):
-                                    line = line[:-1]
-                                grep_object.add_file_exclude_globs(line)
-                    except EnvironmentError as ex:
-                        if not args.no_messages:
-                            print('{}: {}'.format(THIS_FILE_NAME, str(ex)), file=sys.stderr)
-
-        for exclude_glob in args.exclude_dir:
-            grep_object.add_dir_exclude_globs(exclude_glob)
+        grep_object.add_dir_exclude_globs(self._args.exclude_dir)
 
         return True
 
